//process_func =  Module.cwrap('process_img', 'number', ['array','number','array','number','number','number','number','number','number','number','number']);
//width_func = Module.cwrap('width_from_cw4', 'number', ['array','number','number','number']);
//height_func = Module.cwrap('height_from_cw4', 'number', ['array','number','number','number']);
//readfloat_func = Module.cwrap('read_float', 'number', ['array']);

class ThumbDownload {
	constructor(canvas, url, wantW, wantH) {
		this.canvas = canvas;
		this.url = url;
		this.wantW = wantW;
		this.wantH = wantH;
	}
}

var scheduledThumbDownloads = [];
var thumbsDownloading = 0;
var maxThumbDownloads = 4;

function tryNextThumbDownload() {
	while( (scheduledThumbDownloads.length > 0) && (thumbsDownloading < maxThumbDownloads) ) {
		var td = scheduledThumbDownloads.shift();
		thumbsDownloading += 1;
		loadThumb(td.canvas, td.url, td.wantW, td.wantH);
	}
}

function scheduleThumb(canvas, url, wantW, wantH) {
	var td = new ThumbDownload(canvas, url, wantW, wantH);
	scheduledThumbDownloads.push( td );
	tryNextThumbDownload();
}

function discardThumbDownloadQueue() {
	scheduledThumbDownloads = new Array();
}

function loadThumb(canvas, url, wantW, wantH) {
	var oReq = new XMLHttpRequest();
	oReq.framecanvas = canvas;
	oReq.open("GET", url, true);
	oReq.responseType = "arraybuffer";

	oReq.onerror = function (oEvent) {
		thumbsDownloading -= 1;
		tryNextThumbDownload();
	}; // error handler

	oReq.onload = function (oEvent) {
		if(oReq.status!=200) {
			thumbsDownloading -= 1;
			tryNextThumbDownload();
			if(oReq.statusText) {
				console.log('frame.jsp returned status ', oReq.statusText);
			}
		}
		else {
			// check for redirect (getAllResponseHeaders seem to convert names top lowercase)
			if ( oReq.getAllResponseHeaders().toLowerCase().indexOf( "targeturl" ) >= 0 ) {
				loadThumb( canvas, oReq.getResponseHeader( "TargetURL" ), wantW, wantH );
				return;
			}
			thumbsDownloading -= 1;
			tryNextThumbDownload();
			var arrayBuffer = oReq.response; // Note: not oReq.responseText
			if(arrayBuffer && (arrayBuffer.byteLength>40)) {
				var _start = performance.now();
				var cmpArray = new Uint8Array(arrayBuffer);
				var ei4HeaderSize = readShort(cmpArray,6);
				if ( arrayBuffer.byteLength <= (ei4HeaderSize+16) ) {
					return;
				}
				var cmpHeapBytes = new WasmMem()
				cmpHeapBytes.set(cmpArray);
				var _copy1 = performance.now();
				
				let intercept = Module._read_float(cmpHeapBytes.byteOffset+8);
				let slope = Module._read_float(cmpHeapBytes.byteOffset+12);
				let wl_window = Module._read_float(cmpHeapBytes.byteOffset+16);
				let wl_level = Module._read_float(cmpHeapBytes.byteOffset+20);
				
				let targetW = Module._width_from_cw4(cmpHeapBytes.byteOffset+ei4HeaderSize,cmpHeapBytes.length-ei4HeaderSize,wantW,wantH);
				let targetH = Module._height_from_cw4(cmpHeapBytes.byteOffset+ei4HeaderSize,cmpHeapBytes.length-ei4HeaderSize,wantW,wantH);
				
				console.log( '(' , cmpHeapBytes.length-ei4HeaderSize, ') -> ', targetW, '/', targetH, '/', intercept, '/', slope, '/', wl_window ,'/', wl_level);
				
				this.framecanvas.width=targetW;
				this.framecanvas.height=targetH;
				
				var rawHeapBytes = new WasmMem();
				rawHeapBytes.allocate(targetW*targetH*4);
					
				var _copy2 = performance.now();
				
				//let result = Module._gwav4_decompress(cmpHeapBytes.byteOffset, cmpHeapBytes.length, rawHeapBytes.byteOffset, rawHeapBytes.length);
				
				let result = Module._process_img(cmpHeapBytes.byteOffset+ei4HeaderSize, cmpHeapBytes.length-ei4HeaderSize, rawHeapBytes.byteOffset, targetW, targetH,
					1, wl_window, wl_level, slope, intercept, 0);
				
				var _deco = performance.now();
				//window.alert('gwav4_decompress returned ' + result + ' [' + rawHeapBytes[0] + ',' + rawHeapBytes[1] + ']');
				//console.log('times: ', _copy1-_start, ' ', _copy2-_copy1, ' ', _deco-_copy2);
				
				cmpHeapBytes.release();
				
				var ctx = this.framecanvas.getContext('2d');
				var myImageData = ctx.createImageData(targetW, targetH);
				myImageData.data.set(rawHeapBytes.array(),0,targetW*targetH*4);
				
				rawHeapBytes.release();
				
				ctx.putImageData(myImageData,0,0);
				if( this.framecanvas.onLoaded ) {
					this.framecanvas.onLoaded();
				}
			}
		}
	};//loadHandler;
	oReq.send(null);
}
